﻿using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Identity.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using WebApiServer.Models;

namespace WebApiServer.Authentication
{
    public class ApplicationDbContext : IdentityDbContext<ApplicationUser>
    {
        private readonly IConfiguration configuration;
        public DbSet<Event> Events { get; set; }
        public DbSet<EventPointItem> EventPoints { get; set; }
        public DbSet<EventLevel> EventLevels { get; set; }
        public DbSet<Source> Sources { get; set; }
        public DbSet<UserSource> UserSources { get; set; }
        public DbSet<ShopItem> ShopItems { get; set; }
        public DbSet<Transaction> Transactions { get; set; }
        public DbSet<FileModel> Files { get; set; }
        public DbSet<AssetBundleResource> AssetBundleResources { get; set; }
        public DbSet<UserCompletedEvent> UserCompletedEvents { get; set; }
        public DbSet<Setting> Settings { get; set; }
        public ApplicationDbContext(DbContextOptions<ApplicationDbContext> options, IConfiguration configuration) : base(options)
        {
            this.configuration = configuration;
        }
        protected override void OnModelCreating(ModelBuilder builder)
        {
            base.OnModelCreating(builder);

            builder.Entity<ApplicationUser>().ToTable(configuration["TablesPrefix"] + "_Users");
            builder.Entity<IdentityUserRole<string>>().ToTable(configuration["TablesPrefix"] + "_UserRoles");
            builder.Entity<IdentityUserLogin<string>>().ToTable(configuration["TablesPrefix"] + "_UserLogins");
            builder.Entity<IdentityUserClaim<string>>().ToTable(configuration["TablesPrefix"] + "_UserClaims");
            builder.Entity<IdentityRole>().ToTable(configuration["TablesPrefix"] + "_Roles");
            builder.Entity<IdentityRoleClaim<string>>().ToTable(configuration["TablesPrefix"] + "_AspNetRoleClaims");
            builder.Entity<IdentityUserToken<string>>().ToTable(configuration["TablesPrefix"] + "_AspNetUserTokens");
            builder.Entity<UserSource>().ToTable(configuration["TablesPrefix"] + "_UserSources").HasKey(o => new { o.UserId, o.SourceId });
        }

        public override Task<int> SaveChangesAsync(bool acceptAllChangesOnSuccess, CancellationToken cancellationToken = default(CancellationToken))
        {
            var AddedEntities = ChangeTracker.Entries()
                .Where(E => E.State == EntityState.Added)
                .ToList();

            AddedEntities.ForEach(E =>
            {
                try
                {
                    E.Property("CreatedDate").CurrentValue = DateTime.Now;
                }
                catch(Exception e)
                { }
            });

            var EditedEntities = ChangeTracker.Entries()
                .Where(E => E.State == EntityState.Modified)
                .ToList();

            EditedEntities.ForEach(E =>
            {
                E.Property("ModifiedDate").CurrentValue = DateTime.Now;
            });

            return base.SaveChangesAsync(acceptAllChangesOnSuccess, cancellationToken);
        }
    }


    public static class MigrationHelper
    {
        public static bool IsMigrationOperationExecuting()
        {
            var commandLineArguments = Environment.GetCommandLineArgs();
            string[] orderedMigrationArguments = { "migrations", "add" };

            for (var i = 0; i <= commandLineArguments.Length - orderedMigrationArguments.Length; i++)
            {
                if (commandLineArguments.Skip(i).Take(orderedMigrationArguments.Length).SequenceEqual(orderedMigrationArguments))
                    return true;
            }

            return false;
        }
    }
}
