﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Caching.Memory;
using Microsoft.Extensions.Configuration;
using Newtonsoft.Json;
using WebApiServer.Authentication;
using WebApiServer.Models;
using WebApiServer.ViewModels;

namespace WebApiServer.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class SettingsController : Controller
    {
        private readonly UserManager<ApplicationUser> userManager;
        private readonly IConfiguration configuration;
        private readonly ApplicationDbContext dbContext;
        private readonly IMemoryCache memoryCache;
        public SettingsController(UserManager<ApplicationUser> userManager, IConfiguration configuration, ApplicationDbContext dbContext, IMemoryCache memoryCache)
        {
            this.userManager = userManager;
            this.configuration = configuration;
            this.dbContext = dbContext;
            this.memoryCache = memoryCache;
        }

        [Authorize]
        [HttpGet]
        [Route("GetShopItems")]
        public async Task<IActionResult> GetShopItems()
        {
            var events = await dbContext.ShopItems.Where(i=>i.Active).ToListAsync();
            return Ok(events);
        }

        [Route("Edit")]
        public async Task<IActionResult> Edit()
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var settings = await dbContext.Settings.ToListAsync();
                if (settings != null)
                {
                    return View(new EditSettingsModel
                    {
                        Settings = settings,
                    });
                }
                return RedirectToAction("Index", controllerName: "Settings");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }

        [HttpPost]
        [Route("SaveSettings")]
        public async Task<IActionResult> SaveSettings([FromForm] EditSettingsModel model)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                if (model.Settings != null && model.Settings.Count > 0)
                {
                    for (int i = 0; i < model.Settings.Count; i++)
                    {
                        var setting = model.Settings[i];
                        var key = setting.SettingKey;
                        var s = await dbContext.Settings.FirstOrDefaultAsync(r => r.SettingKey == key);
                        if (s != null)
                        {
                            s.SettingValue = setting.SettingValue;
                        }
                        else
                        {
                            dbContext.Settings.Add(new Setting()
                            {
                                SettingKey = setting.SettingKey,
                                SettingValue = setting.SettingValue,
                            });
                        }
                    }

                    await dbContext.SaveChangesAsync();
                    TempData["Message"] = "Settings successfully saved.";
                }
                return RedirectToAction("Edit", controllerName: "Settings");
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }

        public static async Task<string> GetSetting(ApplicationDbContext dbContext, string key)
        {
            var setting = await dbContext.Settings.FirstOrDefaultAsync(s => s.SettingKey == key);
            if (setting != null)
            {
                return setting.SettingValue;
            }
            return null;
        }

    }
}
