﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Caching.Memory;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using Microsoft.Net.Http.Headers;
using Newtonsoft.Json;
using WebApiServer.Authentication;
using WebApiServer.Models;
using WebApiServer.ViewModels;

namespace WebApiServer.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class ShopController : Controller
    {
        private readonly UserManager<ApplicationUser> userManager;
        private readonly IConfiguration configuration;
        private readonly ApplicationDbContext dbContext;
        private readonly IMemoryCache memoryCache;
        private readonly IHttpClientFactory _httpClientFactory;
        private ILogger _logger;
        public ShopController(UserManager<ApplicationUser> userManager, IConfiguration configuration, ApplicationDbContext dbContext, IMemoryCache memoryCache, IHttpClientFactory httpClientFactory, ILoggerFactory loggerFactory)
        {
            this.userManager = userManager;
            this.configuration = configuration;
            this.dbContext = dbContext;
            this.memoryCache = memoryCache;
            this._httpClientFactory = httpClientFactory;
            _logger = loggerFactory.CreateLogger(typeof(TransactionController));
        }

        [Authorize]
        [HttpPost]
        [Route("Claim")]
        public async Task<IActionResult> Claim([FromBody] Request_ClaimBuy model)
        {
            ApplicationUser user = await userManager.FindByNameAsync(HttpContext.User.Identity.Name);
            if (user != null)
            {
                var transactionInfo = await dbContext.Transactions.FirstOrDefaultAsync(e => e.Address == model.Receipt);
                if (transactionInfo == null)
                {
                    var itemInfo = await dbContext.ShopItems.FirstOrDefaultAsync(e => e.Id == model.ItemId);
                    if (itemInfo != null)
                    {
                        using var transaction = dbContext.Database.BeginTransaction();

                        var userSource = await dbContext.UserSources.FirstOrDefaultAsync(s => s.UserId == user.Id && s.SourceId == itemInfo.SourceId);
                        if (userSource != null)
                        {
                            userSource.Value += itemInfo.SourceValue;
                        }
                        else
                        {
                            userSource = new UserSource()
                            {
                                SourceId = itemInfo.SourceId,
                                Value = itemInfo.SourceValue,
                                UserId = user.Id,
                            };
                            await dbContext.UserSources.AddAsync(userSource);
                        }

                        dbContext.Transactions.Add(new Transaction()
                        {
                            UserId = user.Id,
                            Address = model.Receipt,
                            ItemId = model.ItemId,
                            Status = 1,
                        });

                        int result = await dbContext.SaveChangesAsync();
                        if (result > 0)
                        {
                            transaction.Commit();
                            var sources = await (from s in dbContext.Sources
                                                 join us in dbContext.UserSources on new { Id = s.Id, UserId = user.Id } equals new { Id = us.SourceId, UserId = us.UserId } into gj
                                                 from x in gj.DefaultIfEmpty()
                                                 select new
                                                 {
                                                     Name = s.Name,
                                                     Value = (x == null ? 0 : x.Value)
                                                 }).ToListAsync();
                            return Ok(sources);
                        }
                        transaction.Rollback();
                    }
                }
                else
                {
                    var itemInfo = await dbContext.ShopItems.FirstOrDefaultAsync(e => e.Id == transactionInfo.ItemId);
                    if (itemInfo != null)
                    {
                        var sources = await (from s in dbContext.Sources
                                             join us in dbContext.UserSources on new { Id = s.Id, UserId = user.Id } equals new { Id = us.SourceId, UserId = us.UserId } into gj
                                             from x in gj.DefaultIfEmpty()
                                             select new
                                             {
                                                 Name = s.Name,
                                                 Value = (x == null ? 0 : x.Value)
                                             }).ToListAsync();
                        return Ok(sources);
                    }
                }
            }
            return StatusCode(StatusCodes.Status500InternalServerError);
        }
        public async Task<IActionResult> Index(bool confirmed = true, int skip = 0, int take = 50)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var query = (from t in dbContext.Transactions
                             join tuser in dbContext.Users on t.UserId equals tuser.Id
                             join sItem in dbContext.ShopItems on t.ItemId equals sItem.Id
                              where t.Status == (confirmed ? 1 : 0)
                              select new TransactionViewModel
                              {
                                  Id = t.Id,
                                  Status = t.Status == 0?"Created":"Payed",
                                  Address = t.Address,
                                  CreatedDate = DateTime.Now,
                                  ModifiedDate = DateTime.Now,
                                  ItemId = t.ItemId,
                                  UserId = t.UserId,
                                  DisplayName = tuser.DisplayName,
                                  ItemName = sItem.Title,
                                  ItemPrice = sItem.Price,
                              }).OrderBy(r => r.Id);

                ViewData["NextSkip"] = skip + take;
                ViewData["PrevSkip"] = skip - take;

                var rows = await query.Skip(skip).Take(take).ToListAsync();

                return View(rows);
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }
    }
}
