﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Caching.Memory;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using Microsoft.Net.Http.Headers;
using Newtonsoft.Json;
using WebApiServer.Authentication;
using WebApiServer.Models;
using WebApiServer.ViewModels;

namespace WebApiServer.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class TransactionController : Controller
    {
        private readonly UserManager<ApplicationUser> userManager;
        private readonly IConfiguration configuration;
        private readonly ApplicationDbContext dbContext;
        private readonly IMemoryCache memoryCache;
        private readonly IHttpClientFactory _httpClientFactory;
        private ILogger _logger;
        public TransactionController(UserManager<ApplicationUser> userManager, IConfiguration configuration, ApplicationDbContext dbContext, IMemoryCache memoryCache, IHttpClientFactory httpClientFactory, ILoggerFactory loggerFactory)
        {
            this.userManager = userManager;
            this.configuration = configuration;
            this.dbContext = dbContext;
            this.memoryCache = memoryCache;
            this._httpClientFactory = httpClientFactory;
            _logger = loggerFactory.CreateLogger(typeof(TransactionController));
        }

        [Authorize]
        [HttpGet]
        [Route("GetAll")]
        public async Task<IActionResult> GetAll()
        {
            ApplicationUser user = await userManager.FindByNameAsync(HttpContext.User.Identity.Name);
            if (user != null)
            {
                var transactions = await dbContext.Transactions.Where(t => t.Status == 0 && t.UserId == user.Id).OrderByDescending(t => t.Id).Take(20).ToListAsync();
                return Ok(transactions);
            }
            return Unauthorized();
        }
        static string ComputeSha256Hash(string rawData)
        {
            // Create a SHA256   
            using (SHA256 sha256Hash = SHA256.Create())
            {
                // ComputeHash - returns byte array  
                byte[] bytes = sha256Hash.ComputeHash(Encoding.UTF8.GetBytes(rawData));

                // Convert byte array to a string   
                StringBuilder builder = new StringBuilder();
                for (int i = 0; i < bytes.Length; i++)
                {
                    builder.Append(bytes[i].ToString("x2"));
                }
                return builder.ToString();
            }
        }

        public async Task<IActionResult> Index(bool confirmed = true, int skip = 0, int take = 50)
        {
            var username = HttpContext.Session.GetString("Username");
            var role = HttpContext.Session.GetString("Role");

            if (!string.IsNullOrEmpty(username) && role.Equals("Administrator"))
            {
                var query = (from t in dbContext.Transactions
                             join tuser in dbContext.Users on t.UserId equals tuser.Id
                             join sItem in dbContext.ShopItems on t.ItemId equals sItem.Id
                              where t.Status == (confirmed ? 1 : 0)
                              select new TransactionViewModel
                              {
                                  Id = t.Id,
                                  Status = t.Status == 0?"Created":"Payed",
                                  Address = t.Address,
                                  CreatedDate = DateTime.Now,
                                  ModifiedDate = DateTime.Now,
                                  ItemId = t.ItemId,
                                  UserId = t.UserId,
                                  DisplayName = tuser.DisplayName,
                                  ItemName = sItem.Title,
                                  ItemPrice = sItem.Price,
                              }).OrderBy(r => r.Id);

                ViewData["NextSkip"] = skip + take;
                ViewData["PrevSkip"] = skip - take;

                var rows = await query.Skip(skip).Take(take).ToListAsync();

                return View(rows);
            }
            else
            {
                return RedirectToAction("Login", controllerName: "Account");
            }
        }
    }
}
