﻿using Microsoft.AspNetCore.Http;
using System;
using System.Threading.Tasks;

namespace WebApiServer
{
    public class ServerTimingMiddleware
    {
        private static readonly DateTime UnixEpoch = new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc);
        public static long GetCurrentUnixTimestampMillis()
        {
            return (long)(DateTime.UtcNow - UnixEpoch).TotalMilliseconds;
        }
        public static DateTime UnixTimeStampToDateTime(long unixTimeStamp)
        {
            DateTime dtDateTime = new DateTime(1970, 1, 1, 0, 0, 0, 0, System.DateTimeKind.Utc);
            dtDateTime = dtDateTime.AddMilliseconds(unixTimeStamp);
            return dtDateTime;
        }

        private readonly RequestDelegate _next;

        private static Task _completedTask = Task.FromResult<object>(null);

        public ServerTimingMiddleware(RequestDelegate next)
        {
            _next = next ?? throw new ArgumentNullException(nameof(next));
        }

        public Task Invoke(HttpContext context)
        {
            HandleServerTiming(context);

            return _next(context);
        }

        private void HandleServerTiming(HttpContext context)
        {
            context.Response.OnStarting(() => {

                context.Response.Headers.Append("ServerTime", GetCurrentUnixTimestampMillis().ToString());

                return _completedTask;
            });
        }
    }
}
